'use strict';

var react = require('react');

const InitialAsyncState = {
    loading: true,
    result: undefined,
    error: undefined,
};
const defaultSetLoading = _asyncState => InitialAsyncState;
const defaultSetResult = (result, _asyncState) => ({
    loading: false,
    result: result,
    error: undefined,
});
const defaultSetError = (error, _asyncState) => ({
    loading: false,
    result: undefined,
    error: error,
});
const DefaultOptions = {
    initialState: () => InitialAsyncState,
    executeOnMount: true,
    executeOnUpdate: true,
    setLoading: defaultSetLoading,
    setResult: defaultSetResult,
    setError: defaultSetError,
};
const normalizeOptions = (options) => ({
    ...DefaultOptions,
    ...options,
});
const useAsyncState = (options) => {
    const [value, setValue] = react.useState(() => options.initialState());
    return {
        value,
        set: setValue,
        setLoading: () => setValue(options.setLoading(value)),
        setResult: result => setValue(options.setResult(result, value)),
        setError: error => setValue(options.setError(error, value)),
    };
};
const useIsMounted = () => {
    const ref = react.useRef(false);
    react.useEffect(() => {
        ref.current = true;
        return () => {
            ref.current = false;
        };
    }, []);
    return () => ref.current;
};
const useCurrentPromise = () => {
    const ref = react.useRef(null);
    return {
        set: promise => (ref.current = promise),
        get: () => ref.current,
        is: promise => ref.current === promise,
    };
};
// Relaxed interface which accept both async and sync functions
// Accepting sync function is convenient for useAsyncCallback
const useAsyncInternal = (asyncFunction, params, options) => {
    const normalizedOptions = normalizeOptions(options);
    const AsyncState = useAsyncState(normalizedOptions);
    const isMounted = useIsMounted();
    const CurrentPromise = useCurrentPromise();
    // We only want to handle the promise result/error
    // if it is the last operation and the comp is still mounted
    const shouldHandlePromise = (p) => isMounted() && CurrentPromise.is(p);
    const executeAsyncOperation = (...args) => {
        const promise = asyncFunction(...args);
        if (promise instanceof Promise) {
            CurrentPromise.set(promise);
            AsyncState.setLoading();
            promise.then(result => {
                if (shouldHandlePromise(promise)) {
                    AsyncState.setResult(result);
                }
            }, error => {
                if (shouldHandlePromise(promise)) {
                    AsyncState.setError(error);
                }
            });
            return promise;
        }
        else {
            // We allow passing a non-async function (mostly for useAsyncCallback conveniency)
            const syncResult = promise;
            AsyncState.setResult(syncResult);
            return Promise.resolve(syncResult);
        }
    };
    // Keep this outside useEffect, because inside isMounted()
    // will be true as the component is already mounted when it's run
    const isMounting = !isMounted();
    react.useEffect(() => {
        if (isMounting) {
            normalizedOptions.executeOnMount && executeAsyncOperation(...params);
        }
        else {
            normalizedOptions.executeOnUpdate && executeAsyncOperation(...params);
        }
    }, params);
    return {
        ...AsyncState.value,
        set: AsyncState.set,
        execute: executeAsyncOperation,
        currentPromise: CurrentPromise.get(),
    };
};
function useAsync(asyncFunction, params, options) {
    return useAsyncInternal(asyncFunction, params, options);
}
const useAsyncAbortable = (asyncFunction, params, options) => {
    const abortControllerRef = react.useRef();
    // Wrap the original async function and enhance it with abortion login
    const asyncFunctionWrapper = async (...p) => {
        // Cancel previous async call
        if (abortControllerRef.current) {
            abortControllerRef.current.abort();
        }
        // Create/store new abort controller for next async call
        const abortController = new AbortController();
        abortControllerRef.current = abortController;
        try {
            // @ts-ignore // TODO how to type this?
            return await asyncFunction(abortController.signal, ...p);
        }
        finally {
            // Unset abortController ref if response is already there,
            // as it's not needed anymore to try to abort it (would it be no-op?)
            if (abortControllerRef.current === abortController) {
                abortControllerRef.current = undefined;
            }
        }
    };
    return useAsync(asyncFunctionWrapper, params, options);
};
const useAsyncCallback = (asyncFunction) => {
    return useAsyncInternal(asyncFunction, 
    // Hacky but in such case we don't need the params,
    // because async function is only executed manually
    [], {
        executeOnMount: false,
        executeOnUpdate: false,
        initialState: () => ({
            loading: false,
            result: undefined,
            error: undefined,
        }),
    });
};

exports.useAsync = useAsync;
exports.useAsyncAbortable = useAsyncAbortable;
exports.useAsyncCallback = useAsyncCallback;
//# sourceMappingURL=react-async-hook.cjs.development.js.map
